@@ -1,24 +1,22 @@
 from __future__ import division, absolute_import, unicode_literals
 
-from PyQt4 import QtGui
-from PyQt4 import QtCore
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy import QtCore
+from qtpy import QtWidgets
+from qtpy.QtCore import Qt
 
-from cola import core
-from cola import gitcmds
-from cola import hotkeys
-from cola import icons
-from cola import qtutils
-from cola import utils
-from cola.i18n import N_
-from cola.interaction import Interaction
-from cola.models import main
-from cola.models import selection
-from cola.widgets import completion
-from cola.widgets import defs
-from cola.widgets import filetree
-from cola.compat import ustr
+from . import core
+from . import gitcmds
+from . import hotkeys
+from . import icons
+from . import qtutils
+from . import utils
+from .i18n import N_
+from .interaction import Interaction
+from .models import main
+from .models import selection
+from .widgets import completion
+from .widgets import defs
+from .widgets import filetree
 
 
 def run():
@@ -54,7 +52,7 @@ def launch(left=None, right=None, paths=None,
             status, out, err = git.rev_list(left, parents=True, n=1)
             Interaction.log_status(status, out, err)
             if status:
-                raise StandardError('git rev-list command failed')
+                raise OSError('git rev-list command failed')
 
             if len(out.split()) >= 2:
                 # Commit has a parent, so we can take its child as requested
@@ -80,7 +78,7 @@ def diff_commits(parent, a, b):
     dlg = FileDiffDialog(parent, a=a, b=b)
     dlg.show()
     dlg.raise_()
-    return dlg.exec_() == QtGui.QDialog.Accepted
+    return dlg.exec_() == QtWidgets.QDialog.Accepted
 
 
 def diff_expression(parent, expr,
@@ -90,14 +88,14 @@ def diff_expression(parent, expr,
         return dlg
     dlg.show()
     dlg.raise_()
-    return dlg.exec_() == QtGui.QDialog.Accepted
+    return dlg.exec_() == QtWidgets.QDialog.Accepted
 
 
-class FileDiffDialog(QtGui.QDialog):
+class FileDiffDialog(QtWidgets.QDialog):
 
     def __init__(self, parent, a=None, b=None, expr=None, title=None,
                  hide_expr=False):
-        QtGui.QDialog.__init__(self, parent)
+        QtWidgets.QDialog.__init__(self, parent)
         self.setAttribute(Qt.WA_MacMetalStyle)
 
         self.a = a
@@ -133,21 +131,15 @@ class FileDiffDialog(QtGui.QDialog):
                                         self.button_layout)
         self.setLayout(self.main_layout)
 
-        self.connect(self.tree, SIGNAL('itemSelectionChanged()'),
-                     self.tree_selection_changed)
+        self.tree.itemSelectionChanged.connect(self.tree_selection_changed)
+        self.tree.itemDoubleClicked.connect(self.tree_double_clicked)
+        self.tree.up.connect(self.focus_input)
 
-        self.connect(self.tree,
-                     SIGNAL('itemDoubleClicked(QTreeWidgetItem*,int)'),
-                     self.tree_double_clicked)
+        self.expr.textChanged.connect(self.text_changed)
 
-        self.connect(self.expr, SIGNAL('textChanged(QString)'),
-                     self.text_changed)
-        self.connect(self.tree, SIGNAL('up()'), self.focus_input)
-
-        self.connect(self.expr, SIGNAL('activated()'), self.focus_tree)
-        self.connect(self.expr, SIGNAL('down()'), self.focus_tree)
-        self.connect(self.expr, SIGNAL('enter()'), self.focus_tree)
-        self.connect(self.expr, SIGNAL('return()'), self.focus_tree)
+        self.expr.activated.connect(self.focus_tree)
+        self.expr.down.connect(self.focus_tree)
+        self.expr.enter.connect(self.focus_tree)
 
         qtutils.connect_button(self.diff_button, self.diff)
         qtutils.connect_button(self.close_button, self.close)
@@ -165,7 +157,7 @@ class FileDiffDialog(QtGui.QDialog):
         self.expr.setFocus()
 
     def text_changed(self, txt):
-        self.diff_expr = ustr(txt)
+        self.diff_expr = txt
         self.refresh()
 
     def refresh(self):
