@@ -1,36 +1,38 @@
 from __future__ import division, absolute_import, unicode_literals
 
+import collections
 import time
 
-from PyQt4 import QtCore
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy import QtCore
+from qtpy import QtGui
+from qtpy import QtWidgets
+from qtpy.QtCore import Qt
+from qtpy.QtCore import Signal
 
-from cola import gitcfg
-from cola import core
-from cola import utils
-from cola import qtutils
-from cola import version
-from cola import resources
-from cola.git import STDOUT
-from cola.i18n import N_
-from cola.models import main
+from .. import gitcfg
+from .. import gitcmds
+from .. import core
+from .. import icons
+from .. import utils
+from .. import qtutils
+from ..git import STDOUT
+from ..i18n import N_
+from ..models import main
 
 
 # Custom event type for GitRepoInfoEvents
-INFO_EVENT_TYPE = QtCore.QEvent.User + 42
+INFO_EVENT_TYPE = QtCore.QEvent.Type(QtCore.QEvent.registerEventType())
 
 
 class Columns(object):
     """Defines columns in the worktree browser"""
 
-    NAME = 'Name'
-    STATUS = 'Status'
-    AGE = 'Age'
-    MESSAGE = 'Message'
-    AUTHOR = 'Author'
-    ALL = (NAME, STATUS, AGE, MESSAGE, AUTHOR)
+    NAME = 'name'
+    STATUS = 'status'
+    AGE = 'age'
+    MESSAGE = 'message'
+    AUTHOR = 'author'
+    ALL = (NAME, STATUS, MESSAGE, AUTHOR, AGE)
 
     @classmethod
     def text(cls, column):
@@ -38,67 +40,121 @@ class Columns(object):
             return N_('Name')
         elif column == cls.STATUS:
             return N_('Status')
-        elif column == cls.AGE:
-            return N_('Age')
         elif column == cls.MESSAGE:
             return N_('Message')
         elif column == cls.AUTHOR:
             return N_('Author')
+        elif column == cls.AGE:
+            return N_('Age')
         else:
             raise NotImplementedError('Mapping required for "%s"' % column)
 
 
+class GitRepoEntryStore(object):
+
+    entries = {}
+
+    @classmethod
+    def entry(cls, path, parent, runtask):
+        """Return the shared GitRepoEntry for a path."""
+        try:
+            e = cls.entries[path]
+        except KeyError:
+            e = cls.entries[path] = GitRepoEntry(path, parent, runtask)
+        return e
+
+    @classmethod
+    def remove(cls, path):
+        try:
+            del cls.entries[path]
+        except KeyError:
+            pass
+
+
+def _item_path(item):
+    """Return the item's path"""
+    try:
+        path = item.path
+    except AttributeError:
+        # the root QStandardItem does not have a 'path' attribute
+        path = ''
+    return path
+
+
 class GitRepoModel(QtGui.QStandardItemModel):
     """Provides an interface into a git repository for browsing purposes."""
 
+    updated = Signal()
+    restore = Signal()
+
     def __init__(self, parent):
         QtGui.QStandardItemModel.__init__(self, parent)
-        self._interesting_paths = self._get_paths()
-        self._known_paths = set()
 
-        self.connect(self, SIGNAL('updated'), self._updated_callback)
+        self.entries = {}
+        self._runtask = qtutils.RunTask(parent=parent)
+        self._parent = parent
+        self._interesting_paths = set()
+        self._interesting_files = set()
+        self._known_paths = set()
+        self._dir_entries = {}
+        self._dir_rows = collections.defaultdict(int)
+
+        self.updated.connect(self.refresh, type=Qt.QueuedConnection)
+
         model = main.model()
         model.add_observer(model.message_updated, self._model_updated)
-        self._dir_rows = {}
-        self.setColumnCount(len(Columns.ALL))
-        for idx, header in enumerate(Columns.ALL):
-            text = Columns.text(header)
-            self.setHeaderData(idx, Qt.Horizontal, QtCore.QVariant(text))
 
-        self._direntries = {'': self.invisibleRootItem()}
-        self._initialize()
+        self.file_icon = icons.file_text()
+        self.dir_icon = icons.directory()
 
     def mimeData(self, indexes):
         paths = qtutils.paths_from_indexes(self, indexes,
                                            item_type=GitRepoNameItem.TYPE)
         return qtutils.mimedata_from_paths(paths)
 
-    def _create_column(self, col, path):
+    def mimeTypes(self):
+        return qtutils.path_mimetypes()
+
+    def clear(self):
+        super(GitRepoModel, self).clear()
+        self.entries.clear()
+
+    def row(self, path, create=True):
+        try:
+            row = self.entries[path]
+        except KeyError:
+            if create:
+                row = self.entries[path] = [self.create_column(c, path)
+                                            for c in Columns.ALL]
+            else:
+                row = None
+        return row
+
+    def create_column(self, col, path):
         """Creates a StandardItem for use in a treeview cell."""
         # GitRepoNameItem is the only one that returns a custom type(),
         # so we use to infer selections.
         if col == Columns.NAME:
-            return GitRepoNameItem(path)
-        return GitRepoItem(col, path)
-
-    def _create_row(self, path):
-        """Return a list of items representing a row."""
-        return [self._create_column(c, path) for c in Columns.ALL]
+            item = GitRepoNameItem(path, self._parent, self._runtask)
+        else:
+            item = GitRepoItem(col, path, self._parent, self._runtask)
+        return item
 
     def _add_file(self, parent, path, insert=False):
         """Add a file entry to the model."""
 
+        self._known_paths.add(path)
+
         # Create model items
-        row_items = self._create_row(path)
+        row_items = self.row(path)
 
         # Use a standard file icon for the name field
-        row_items[0].setIcon(qtutils.file_icon())
+        row_items[0].setIcon(self.file_icon)
 
         if not insert:
             # Add file paths at the end of the list
             parent.appendRow(row_items)
             self.entry(path).update_name()
-            self._known_paths.add(path)
             return
         # Entries exist so try to find an a good insertion point
         done = False
@@ -115,23 +171,23 @@ class GitRepoModel(QtGui.QStandardItemModel):
         if not done:
             parent.appendRow(row_items)
         self.entry(path).update_name()
-        self._known_paths.add(path)
 
     def add_directory(self, parent, path):
         """Add a directory entry to the model."""
 
         # Create model items
-        row_items = self._create_row(path)
+        row_items = self.row(path)
 
         # Use a standard directory icon
-        row_items[0].setIcon(qtutils.dir_icon())
+        row_items[0].setIcon(self.dir_icon)
 
         # Insert directories before file paths
         # TODO: have self._dir_rows's keys based on something less flaky than
         # QStandardItem instances.
-        row = self._dir_rows.setdefault(id(parent), 0)
+        parent_path = _item_path(parent)
+        row = self._dir_rows[parent_path]
         parent.insertRow(row, row_items)
-        self._dir_rows[id(parent)] += 1
+        self._dir_rows[parent_path] += 1
 
         # Update the 'name' column for this entry
         self.entry(path).update_name()
@@ -143,39 +199,62 @@ class GitRepoModel(QtGui.QStandardItemModel):
         """Return True if path has a status."""
         return path in self._interesting_paths
 
-    def _get_paths(self):
+    def get_paths(self, files=None):
         """Return paths of interest; e.g. paths with a status."""
+        if files is None:
+            files = self.get_files()
+        return utils.add_parents(files)
+
+    def get_files(self):
         model = main.model()
-        paths = model.staged + model.unstaged
-        return utils.add_parents(paths)
+        return set(model.staged + model.unstaged)
 
     def _model_updated(self):
         """Observes model changes and updates paths accordingly."""
-        self.emit(SIGNAL('updated'))
+        self.updated.emit()
 
-    def _updated_callback(self):
+    def refresh(self):
+        old_files = self._interesting_files
         old_paths = self._interesting_paths
-        new_paths = self._get_paths()
-        for path in new_paths.union(old_paths):
-            if path not in self._known_paths:
-                continue
+        new_files = self.get_files()
+        new_paths = self.get_paths(files=new_files)
+
+        if new_files != old_files or not old_paths:
+            self.clear()
+            self._initialize()
+            self.restore.emit()
+
+        # Existing items
+        for path in sorted(new_paths.union(old_paths)):
             self.entry(path).update()
 
+        self._interesting_files = new_files
         self._interesting_paths = new_paths
 
     def _initialize(self):
-        """Iterate over the cola model and create GitRepoItems."""
-        for path in main.model().everything():
+
+        self.setColumnCount(len(Columns.ALL))
+        for idx, header in enumerate(Columns.ALL):
+            text = Columns.text(header)
+            self.setHeaderData(idx, Qt.Horizontal, text)
+
+        self._entries = {}
+        self._dir_rows = collections.defaultdict(int)
+        self._known_paths = set()
+        self._dir_entries = {'': self.invisibleRootItem()}
+        self._interesting_files = files = self.get_files()
+        self._interesting_paths = self.get_paths(files=files)
+        for path in gitcmds.all_files():
             self.add_file(path)
 
     def add_file(self, path, insert=False):
         """Add a file to the model."""
         dirname = utils.dirname(path)
-        if dirname in self._direntries:
-            parent = self._direntries[dirname]
+        if dirname in self._dir_entries:
+            parent = self._dir_entries[dirname]
         else:
-            parent = self._create_dir_entry(dirname, self._direntries)
-            self._direntries[dirname] = parent
+            parent = self._create_dir_entry(dirname, self._dir_entries)
+            self._dir_entries[dirname] = parent
         self._add_file(parent, path, insert=insert)
 
     def _create_dir_entry(self, dirname, direntries):
@@ -193,9 +272,9 @@ class GitRepoModel(QtGui.QStandardItemModel):
         for entry in entries:
             curdir_append(entry)
             path = '/'.join(curdir)
-            if path in direntries:
+            try:
                 parent = direntries[path]
-            else:
+            except KeyError:
                 grandparent = parent
                 parent = self_add_directory(grandparent, path)
                 direntries[path] = parent
@@ -203,59 +282,7 @@ class GitRepoModel(QtGui.QStandardItemModel):
 
     def entry(self, path):
         """Return the GitRepoEntry for a path."""
-        return GitRepoEntryManager.entry(path)
-
-
-class GitRepoEntryManager(object):
-    """
-    Provides access to static instances of GitRepoEntry and model data.
-    """
-    static_entries = {}
-
-    @classmethod
-    def entry(cls, path, _static_entries=static_entries):
-        """Return a static instance of a GitRepoEntry."""
-        try:
-            e = _static_entries[path]
-        except KeyError:
-            e = _static_entries[path] = GitRepoEntry(path)
-        return e
-
-
-class TaskRunner(object):
-    """Manages QRunnable task instances to avoid python's garbage collector
-
-    When PyQt stops referencing a QRunnable instance Python cleans it up
-    which leads to segfaults, e.g. the dreaded "C++ object has gone away".
-
-    This class keeps track of tasks and cleans up references to them as they
-    complete.
-
-    """
-    singleton = None
-
-    @classmethod
-    def instance(cls):
-        if cls.singleton is None:
-            cls.singleton = TaskRunner()
-        return cls.singleton
-
-    def __init__(self):
-        self.tasks = set()
-        self.threadpool = QtCore.QThreadPool.globalInstance()
-        self.notifier = QtCore.QObject()
-        self.notifier.connect(self.notifier, SIGNAL('task_done'), self.task_done)
-
-    def run(self, task):
-        self.tasks.add(task)
-        self.threadpool.start(task)
-
-    def task_done(self, task):
-        if task in self.tasks:
-            self.tasks.remove(task)
-
-    def cleanup_task(self, task):
-        self.notifier.emit(SIGNAL('task_done'), task)
+        return GitRepoEntryStore.entry(path, self._parent, self._runtask)
 
 
 class GitRepoEntry(QtCore.QObject):
@@ -265,45 +292,49 @@ class GitRepoEntry(QtCore.QObject):
     Emits signal names matching those defined in Columns.
 
     """
-    def __init__(self, path):
-        QtCore.QObject.__init__(self)
+    name = Signal(object)
+    status = Signal(object)
+    author = Signal(object)
+    message = Signal(object)
+    age = Signal(object)
+
+    def __init__(self, path, parent, runtask):
+        QtCore.QObject.__init__(self, parent)
         self.path = path
+        self.runtask = runtask
 
     def update_name(self):
         """Emits a signal corresponding to the entry's name."""
         # 'name' is cheap to calculate so simply emit a signal
-        self.emit(SIGNAL(Columns.NAME), utils.basename(self.path))
+        self.name.emit(utils.basename(self.path))
         if '/' not in self.path:
             self.update()
 
     def update(self):
         """Starts a GitRepoInfoTask to calculate info for entries."""
         # GitRepoInfoTask handles expensive lookups
-        task = GitRepoInfoTask(self.path)
-        TaskRunner.instance().run(task)
+        task = GitRepoInfoTask(self.path, self, self.runtask)
+        self.runtask.start(task)
 
     def event(self, e):
         """Receive GitRepoInfoEvents and emit corresponding Qt signals."""
         if e.type() == INFO_EVENT_TYPE:
             e.accept()
-            self.emit(SIGNAL(e.signal), *e.data)
+            signal = getattr(self, e.signal)
+            signal.emit(*e.data)
             return True
         return QtCore.QObject.event(self, e)
 
 
-# Support older versions of PyQt
-if version.check('pyqt_qrunnable', QtCore.PYQT_VERSION_STR):
-    QRunnable = QtCore.QRunnable
-else:
-    class QRunnable(object):
-        pass
-
-class GitRepoInfoTask(QRunnable):
+class GitRepoInfoTask(qtutils.Task):
     """Handles expensive git lookups for a path."""
-    def __init__(self, path):
-        QRunnable.__init__(self)
+
+    def __init__(self, path, parent, runtask):
+        qtutils.Task.__init__(self, parent)
         self.path = path
-        self._cfg = gitcfg.instance()
+        self._parent = parent
+        self._runtask = runtask
+        self._cfg = gitcfg.current()
         self._data = {}
 
     def data(self, key):
@@ -315,10 +346,9 @@ class GitRepoInfoTask(QRunnable):
         """
         if not self._data:
             log_line = main.model().git.log('-1', '--', self.path,
-                                            M=True,
-                                            all=False,
                                             no_color=True,
-                                            pretty='format:%ar%x01%s%x01%an'
+                                            pretty=r'format:%ar%x01%s%x01%an',
+                                            _readonly=True
                                             )[STDOUT]
             if log_line:
                 log_line = log_line
@@ -368,39 +398,39 @@ class GitRepoInfoTask(QRunnable):
         upstream_changed = utils.add_parents(model.upstream_changed)
 
         if self.path in unmerged:
-            return (resources.icon('modified.png'), N_('Unmerged'))
-        if self.path in modified and self.path in staged:
-            return (resources.icon('partial.png'), N_('Partially Staged'))
-        if self.path in modified:
-            return (resources.icon('modified.png'), N_('Modified'))
-        if self.path in staged:
-            return (resources.icon('staged.png'), N_('Staged'))
-        if self.path in upstream_changed:
-            return (resources.icon('upstream.png'), N_('Changed Upstream'))
-        if self.path in untracked:
-            return (None, '?')
-        return (None, '')
+            status = (icons.modified_name(), N_('Unmerged'))
+        elif self.path in modified and self.path in staged:
+            status = (icons.partial_name(), N_('Partially Staged'))
+        elif self.path in modified:
+            status = (icons.modified_name(), N_('Modified'))
+        elif self.path in staged:
+            status = (icons.staged_name(), N_('Staged'))
+        elif self.path in upstream_changed:
+            status = (icons.upstream_name(), N_('Changed Upstream'))
+        elif self.path in untracked:
+            status = (None, '?')
+        else:
+            status = (None, '')
+        return status
 
-    def run(self):
+    def task(self):
         """Perform expensive lookups and post corresponding events."""
-        app = QtGui.QApplication.instance()
-        entry = GitRepoEntryManager.entry(self.path)
+        app = QtWidgets.QApplication.instance()
+        entry = GitRepoEntryStore.entry(self.path, self._parent, self._runtask)
         app.postEvent(entry,
-                GitRepoInfoEvent(Columns.MESSAGE, self.data('message')))
+                      GitRepoInfoEvent(Columns.MESSAGE, self.data('message')))
         app.postEvent(entry,
-                GitRepoInfoEvent(Columns.AGE, self.data('date')))
+                      GitRepoInfoEvent(Columns.AGE, self.data('date')))
         app.postEvent(entry,
-                GitRepoInfoEvent(Columns.AUTHOR, self.data('author')))
+                      GitRepoInfoEvent(Columns.AUTHOR, self.data('author')))
         app.postEvent(entry,
-                GitRepoInfoEvent(Columns.STATUS, self.status()))
-
-        TaskRunner.instance().cleanup_task(self)
+                      GitRepoInfoEvent(Columns.STATUS, self.status()))
 
 
 class GitRepoInfoEvent(QtCore.QEvent):
     """Transport mechanism for communicating from a GitRepoInfoTask."""
     def __init__(self, signal, *data):
-        QtCore.QEvent.__init__(self, QtCore.QEvent.User + 1)
+        QtCore.QEvent.__init__(self, INFO_EVENT_TYPE)
         self.signal = signal
         self.data = data
 
@@ -417,15 +447,19 @@ class GitRepoItem(QtGui.QStandardItem):
     One is created for each column -- Name, Status, Age, etc.
 
     """
-    def __init__(self, column, path):
+    def __init__(self, column, path, parent, runtask):
         QtGui.QStandardItem.__init__(self)
+        self.path = path
+        self.runtask = runtask
+        self.cached = False
         self.setDragEnabled(False)
         self.setEditable(False)
-        entry = GitRepoEntryManager.entry(path)
+        entry = GitRepoEntryStore.entry(path, parent, runtask)
         if column == Columns.STATUS:
-            QtCore.QObject.connect(entry, SIGNAL(column), self.set_status)
+            entry.status.connect(self.set_status, type=Qt.QueuedConnection)
         else:
-            QtCore.QObject.connect(entry, SIGNAL(column), self.setText)
+            signal = getattr(entry, column)
+            signal.connect(self.setText, type=Qt.QueuedConnection)
 
     def set_status(self, data):
         icon, txt = data
@@ -438,11 +472,10 @@ class GitRepoItem(QtGui.QStandardItem):
 
 class GitRepoNameItem(GitRepoItem):
     """Subclass GitRepoItem to provide a custom type()."""
-    TYPE = QtGui.QStandardItem.UserType + 1
+    TYPE = QtGui.QStandardItem.ItemType(QtGui.QStandardItem.UserType + 1)
 
-    def __init__(self, path):
-        GitRepoItem.__init__(self, Columns.NAME, path)
-        self.path = path
+    def __init__(self, path, parent, runtask):
+        GitRepoItem.__init__(self, Columns.NAME, path, parent, runtask)
         self.setDragEnabled(True)
 
     def type(self):
