@@ -1,36 +1,34 @@
 """Provides widgets related to bookmarks"""
-
 from __future__ import division, absolute_import, unicode_literals
-
 import os
 
-from PyQt4 import QtCore
-from PyQt4 import QtGui
-from PyQt4.QtCore import SIGNAL
+from qtpy import QtCore
+from qtpy import QtWidgets
+from qtpy.QtCore import Signal
 
-from cola import cmds
-from cola import core
-from cola import git
-from cola import gitcfg
-from cola import hotkeys
-from cola import icons
-from cola import qtutils
-from cola import utils
-from cola.i18n import N_
-from cola.models import prefs
-from cola.settings import Settings
-from cola.widgets import defs
-from cola.widgets import standard
+from .. import cmds
+from .. import core
+from .. import git
+from .. import gitcfg
+from .. import hotkeys
+from .. import icons
+from .. import qtutils
+from .. import utils
+from ..i18n import N_
+from ..models import prefs
+from ..settings import Settings
+from ..widgets import defs
+from ..widgets import standard
 
 
 BOOKMARKS = 0
 RECENT_REPOS = 1
 
 
-class BookmarksWidget(QtGui.QWidget):
+class BookmarksWidget(QtWidgets.QWidget):
 
     def __init__(self, style=BOOKMARKS, parent=None):
-        QtGui.QWidget.__init__(self, parent)
+        QtWidgets.QWidget.__init__(self, parent)
 
         self.style = style
         self.settings = Settings()
@@ -62,7 +60,7 @@ class BookmarksWidget(QtGui.QWidget):
         self.main_layout = qtutils.vbox(defs.no_margin, defs.spacing, self.tree)
         self.setLayout(self.main_layout)
 
-        self.corner_widget = QtGui.QWidget(self)
+        self.corner_widget = QtWidgets.QWidget(self)
         self.corner_widget.setLayout(self.button_layout)
         titlebar = parent.titleBarWidget()
         titlebar.add_corner_widget(self.corner_widget)
@@ -71,8 +69,8 @@ class BookmarksWidget(QtGui.QWidget):
         qtutils.connect_button(self.delete_button, self.tree.delete_bookmark)
         qtutils.connect_button(self.open_button, self.tree.open_repo)
 
-        self.connect(self.tree, SIGNAL('itemSelectionChanged()'),
-                     self.tree_item_selection_changed)
+        item_selection_changed = self.tree_item_selection_changed
+        self.tree.itemSelectionChanged.connect(item_selection_changed)
 
         QtCore.QTimer.singleShot(0, self.reload_bookmarks)
 
@@ -86,51 +84,49 @@ class BookmarksWidget(QtGui.QWidget):
         self.button_group.setEnabled(enabled)
 
     def connect_to(self, other):
-        self.connect(self.tree, SIGNAL('default_changed()'), other.tree.refresh)
-        self.connect(other.tree, SIGNAL('default_changed()'), self.tree.refresh)
+        self.tree.default_changed.connect(other.tree.refresh)
+        other.tree.default_changed.connect(self.tree.refresh)
 
 
 class BookmarksTreeWidget(standard.TreeWidget):
+    default_changed = Signal()
 
     def __init__(self, style, settings, parent=None):
         standard.TreeWidget.__init__(self, parent=parent)
         self.style = style
         self.settings = settings
 
-        self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
         self.setHeaderHidden(True)
 
-        self.open_action = qtutils.add_action(self,
-                N_('Open'), self.open_repo, hotkeys.OPEN, *hotkeys.ACCEPT)
+        self.open_action = qtutils.add_action(
+                self, N_('Open'), self.open_repo, hotkeys.OPEN, *hotkeys.ACCEPT)
 
-        self.open_new_action = qtutils.add_action(self,
-                N_('Open in New Window'), self.open_new_repo, hotkeys.NEW)
+        self.open_new_action = qtutils.add_action(
+                self, N_('Open in New Window'), self.open_new_repo, hotkeys.NEW)
 
-        self.set_default_repo_action = qtutils.add_action(self,
-                N_('Set Default Repository'), self.set_default_repo)
+        self.set_default_repo_action = qtutils.add_action(
+                self, N_('Set Default Repository'), self.set_default_repo)
 
-        self.clear_default_repo_action = qtutils.add_action(self,
-                N_('Clear Default Repository'), self.clear_default_repo)
+        self.clear_default_repo_action = qtutils.add_action(
+                self, N_('Clear Default Repository'), self.clear_default_repo)
 
-        self.open_default_action = qtutils.add_action(self,
-                cmds.OpenDefaultApp.name(), self.open_default,
+        self.open_default_action = qtutils.add_action(
+                self, cmds.OpenDefaultApp.name(), self.open_default,
                 hotkeys.PRIMARY_ACTION)
 
-        self.launch_editor_action = qtutils.add_action(self,
-                cmds.Edit.name(), self.launch_editor, hotkeys.EDIT)
+        self.launch_editor_action = qtutils.add_action(
+                self, cmds.Edit.name(), self.launch_editor, hotkeys.EDIT)
 
-        self.launch_terminal_action = qtutils.add_action(self,
-                cmds.LaunchTerminal.name(), self.launch_terminal,
+        self.launch_terminal_action = qtutils.add_action(
+                self, cmds.LaunchTerminal.name(), self.launch_terminal,
                 hotkeys.TERMINAL)
 
-        self.copy_action = qtutils.add_action(self,
-                N_('Copy'), self.copy, hotkeys.COPY)
+        self.copy_action = qtutils.add_action(
+                self, N_('Copy'), self.copy, hotkeys.COPY)
 
-        self.connect(self, SIGNAL('itemSelectionChanged()'),
-                     self.item_selection_changed)
-
-        self.connect(self, SIGNAL('itemDoubleClicked(QTreeWidgetItem*,int)'),
-                     self.tree_double_clicked)
+        self.itemSelectionChanged.connect(self.item_selection_changed)
+        self.itemDoubleClicked.connect(self.tree_double_clicked)
 
         self.action_group = utils.Group(self.open_action,
                                         self.open_new_action,
@@ -161,7 +157,7 @@ class BookmarksTreeWidget(standard.TreeWidget):
         self.addTopLevelItems(items)
 
     def contextMenuEvent(self, event):
-        menu = QtGui.QMenu(self)
+        menu = qtutils.create_menu(N_('Actions'), self)
         menu.addAction(self.open_action)
         menu.addAction(self.open_new_action)
         menu.addAction(self.open_default_action)
@@ -195,11 +191,11 @@ class BookmarksTreeWidget(standard.TreeWidget):
     def set_default_item(self, item):
         cmds.do(cmds.SetDefaultRepo, item.path)
         self.refresh()
-        self.emit(SIGNAL('default_changed()'))
+        self.default_changed.emit()
 
     def clear_default_repo(self):
         self.apply_fn(self.clear_default_item)
-        self.emit(SIGNAL('default_changed()'))
+        self.default_changed.emit()
 
     def clear_default_item(self, item):
         cmds.do(cmds.SetDefaultRepo, None)
@@ -277,10 +273,10 @@ class BuildItem(object):
         return BookmarksTreeWidgetItem(path, icon, is_default)
 
 
-class BookmarksTreeWidgetItem(QtGui.QTreeWidgetItem):
+class BookmarksTreeWidgetItem(QtWidgets.QTreeWidgetItem):
 
     def __init__(self, path, icon, is_default):
-        QtGui.QTreeWidgetItem.__init__(self)
+        QtWidgets.QTreeWidgetItem.__init__(self)
         self.path = path
         self.is_default = is_default
 
